@@ -133,6 +133,9 @@ fitme <- function(formula,data, ## matches minimal call of HLfit
   oricall <- match.call(expand.dots=TRUE) ## mc including dotlist
   oricall$"control.HLfit" <- eval(oricall$control.HLfit, parent.frame()) # to evaluate variables in the formula_env, otherwise there are bugs in waiting 
   oricall$fixed <- eval(oricall$fixed, parent.frame()) # allows modif in post-fit code (cf get_HLCorcall) 
+  oricall$init <- eval(oricall$init, parent.frame()) # allows modif in post-fit code (cf get_HLCorcall). Better way ? One should be in principle able 
+                                                     # to provide the arguments again to the post-fit call => argument o post-fit fn to provide control of eval envir.
+                                                     # cf also alternative strategy of trying 3 envirs in lme4:::update.merMod(), incl. sys.frames()[[1]]
   mc <- oricall
   #
   if ( ! is.null(weights.form)) {
